// This file was generated by [ts-rs](https://github.com/Aleph-Alpha/ts-rs). Do not edit this file manually.
import type { DataType } from "./DataType";
import type { DomainConstraint } from "./DomainConstraint";
import type { Expression } from "./Expression";
import type { TypeAttribute } from "./TypeAttribute";

/**
 * Type definition
 */
export type TypeDefinition = { "Enum": Array<string> } | { "Composite": Array<TypeAttribute> } | { "Range": { subtype: DataType, subtype_diff: string | null, canonical: string | null, } } | { "Base": { input: string, output: string, internallength: number | null, } } | { "Domain": { base_type: DataType, default: Expression | null, constraints: Array<DomainConstraint>, } };
