// This file was generated by [ts-rs](https://github.com/Aleph-Alpha/ts-rs). Do not edit this file manually.
import type { Expression } from "./Expression";

/**
 * EncryptRaw
 */
export type EncryptRaw = { this: Expression, key: Expression | null, iv: Expression | null, aad: Expression | null, encryption_method: Expression | null, };
