// This file was generated by [ts-rs](https://github.com/Aleph-Alpha/ts-rs). Do not edit this file manually.

/**
 * SQL data access characteristics for functions
 */
export type SqlDataAccess = "NoSql" | "ContainsSql" | "ReadsSqlData" | "ModifiesSqlData";
