// This file was generated by [ts-rs](https://github.com/Aleph-Alpha/ts-rs). Do not edit this file manually.
import type { Identifier } from "./Identifier";

/**
 * DROP DATABASE statement
 */
export type DropDatabase = { name: Identifier, if_exists: boolean, };
