// This file was generated by [ts-rs](https://github.com/Aleph-Alpha/ts-rs). Do not edit this file manually.
import type { Identifier } from "./Identifier";

/**
 * Principal in GRANT/REVOKE (user, role, etc.)
 */
export type GrantPrincipal = { 
/**
 * The name of the principal
 */
name: Identifier, 
/**
 * Whether prefixed with ROLE keyword
 */
is_role: boolean, 
/**
 * Whether prefixed with GROUP keyword (Redshift)
 */
is_group: boolean, };
