// This file was generated by [ts-rs](https://github.com/Aleph-Alpha/ts-rs). Do not edit this file manually.

/**
 * Referential action for foreign keys
 */
export type ReferentialAction = "Cascade" | "SetNull" | "SetDefault" | "Restrict" | "NoAction";
