// This file was generated by [ts-rs](https://github.com/Aleph-Alpha/ts-rs). Do not edit this file manually.

/**
 * Trigger timing (BEFORE, AFTER, INSTEAD OF)
 */
export type TriggerTiming = "Before" | "After" | "InsteadOf";
