// This file was generated by [ts-rs](https://github.com/Aleph-Alpha/ts-rs). Do not edit this file manually.
import type { Expression } from "./Expression";

/**
 * SUBSTRING function
 */
export type SubstringFunc = { this: Expression, start: Expression, length: Expression | null, 
/**
 * Whether SQL standard FROM/FOR syntax was used (true) vs comma-separated (false)
 */
from_for_syntax: boolean, };
