// This file was generated by [ts-rs](https://github.com/Aleph-Alpha/ts-rs). Do not edit this file manually.
import type { Ordered } from "./Ordered";

/**
 * Oracle KEEP clause for aggregate functions
 * Syntax: aggregate_function KEEP (DENSE_RANK FIRST|LAST ORDER BY column [ASC|DESC])
 */
export type Keep = { 
/**
 * true = FIRST, false = LAST
 */
first: boolean, 
/**
 * ORDER BY clause inside KEEP
 */
order_by: Array<Ordered>, };
