// This file was generated by [ts-rs](https://github.com/Aleph-Alpha/ts-rs). Do not edit this file manually.
import type { Expression } from "./Expression";

/**
 * CHAR/CHR function with multiple args and optional USING charset
 * e.g., CHAR(77, 77.3, '77.3' USING utf8mb4)
 * e.g., CHR(187 USING NCHAR_CS) -- Oracle
 */
export type CharFunc = { args: Array<Expression>, charset?: string | null, 
/**
 * Original function name (CHAR or CHR), defaults to CHAR
 */
name?: string | null, };
