// This file was generated by [ts-rs](https://github.com/Aleph-Alpha/ts-rs). Do not edit this file manually.
import type { Expression } from "./Expression";

/**
 * JSONExtractQuote
 */
export type JSONExtractQuote = { option: Expression | null, scalar: Expression | null, };
