// This file was generated by [ts-rs](https://github.com/Aleph-Alpha/ts-rs). Do not edit this file manually.
import type { Expression } from "./Expression";

/**
 * SHA2Digest
 */
export type SHA2Digest = { this: Expression, length: bigint | null, };
