// This file was generated by [ts-rs](https://github.com/Aleph-Alpha/ts-rs). Do not edit this file manually.
import type { Expression } from "./Expression";
import type { Ordered } from "./Ordered";

/**
 * WITHIN GROUP clause (for ordered-set aggregate functions)
 */
export type WithinGroup = { 
/**
 * The aggregate function (LISTAGG, PERCENTILE_CONT, etc.)
 */
this: Expression, 
/**
 * The ORDER BY clause within the group
 */
order_by: Array<Ordered>, };
