// This file was generated by [ts-rs](https://github.com/Aleph-Alpha/ts-rs). Do not edit this file manually.

/**
 * Sequence bound (value or NO MINVALUE/NO MAXVALUE)
 */
export type SequenceBound = { "Value": bigint } | "None";
