// This file was generated by [ts-rs](https://github.com/Aleph-Alpha/ts-rs). Do not edit this file manually.
import type { Expression } from "./Expression";

/**
 * AlterColumn
 */
export type AlterColumn = { this: Expression, dtype: Expression | null, collate: Expression | null, using: Expression | null, default: Expression | null, drop: Expression | null, comment: Expression | null, allow_null: Expression | null, visible: Expression | null, rename_to: Expression | null, };
