// This file was generated by [ts-rs](https://github.com/Aleph-Alpha/ts-rs). Do not edit this file manually.
import type { Expression } from "./Expression";

/**
 * MergeTreeTTLAction
 */
export type MergeTreeTTLAction = { this: Expression, delete: Expression | null, recompress: Expression | null, to_disk: Expression | null, to_volume: Expression | null, };
