// This file was generated by [ts-rs](https://github.com/Aleph-Alpha/ts-rs). Do not edit this file manually.

export type IntervalUnit = "Year" | "Quarter" | "Month" | "Week" | "Day" | "Hour" | "Minute" | "Second" | "Millisecond" | "Microsecond";
