// This file was generated by [ts-rs](https://github.com/Aleph-Alpha/ts-rs). Do not edit this file manually.
import type { Expression } from "./Expression";

/**
 * IndexColumnConstraint
 */
export type IndexColumnConstraint = { this: Expression | null, expressions: Array<Expression>, kind: string | null, index_type: Expression | null, options: Array<Expression>, expression: Expression | null, granularity: Expression | null, };
