// This file was generated by [ts-rs](https://github.com/Aleph-Alpha/ts-rs). Do not edit this file manually.
import type { Expression } from "./Expression";

/**
 * MultitableInserts
 */
export type MultitableInserts = { expressions: Array<Expression>, kind: string, source: Expression | null, 
/**
 * Leading comments before the statement
 */
leading_comments: Array<string>, };
