// This file was generated by [ts-rs](https://github.com/Aleph-Alpha/ts-rs). Do not edit this file manually.
import type { Expression } from "./Expression";

/**
 * DeclareItem
 */
export type DeclareItem = { this: Expression, kind: string | null, default: Expression | null, has_as: boolean, 
/**
 * BigQuery: additional variable names in multi-variable DECLARE (DECLARE X, Y, Z INT64)
 */
additional_names?: Array<Expression>, };
