// This file was generated by [ts-rs](https://github.com/Aleph-Alpha/ts-rs). Do not edit this file manually.

/**
 * Row output semantics for MATCH_RECOGNIZE
 */
export type MatchRecognizeRows = "OneRowPerMatch" | "AllRowsPerMatch" | "AllRowsPerMatchShowEmptyMatches" | "AllRowsPerMatchOmitEmptyMatches" | "AllRowsPerMatchWithUnmatchedRows";
