// This file was generated by [ts-rs](https://github.com/Aleph-Alpha/ts-rs). Do not edit this file manually.
import type { DataType } from "./DataType";
import type { Expression } from "./Expression";

/**
 * OpenJSONColumnDef
 */
export type OpenJSONColumnDef = { this: Expression, kind: string, path: Expression | null, as_json: Expression | null, 
/**
 * The parsed data type for proper generation
 */
data_type?: DataType | null, };
