// This file was generated by [ts-rs](https://github.com/Aleph-Alpha/ts-rs). Do not edit this file manually.
import type { Expression } from "./Expression";

/**
 * RefreshTriggerProperty - Doris REFRESH clause for materialized views
 * e.g., REFRESH COMPLETE ON MANUAL, REFRESH AUTO ON SCHEDULE EVERY 5 MINUTE
 */
export type RefreshTriggerProperty = { 
/**
 * Method: COMPLETE or AUTO
 */
method: string, 
/**
 * Trigger kind: MANUAL, COMMIT, or SCHEDULE
 */
kind: string | null, 
/**
 * For SCHEDULE: EVERY n (the number)
 */
every: Expression | null, 
/**
 * For SCHEDULE: the time unit (MINUTE, HOUR, DAY, etc.)
 */
unit: string | null, 
/**
 * For SCHEDULE: STARTS 'datetime'
 */
starts: Expression | null, };
