// This file was generated by [ts-rs](https://github.com/Aleph-Alpha/ts-rs). Do not edit this file manually.
import type { Expression } from "./Expression";

/**
 * Variable argument function
 */
export type VarArgFunc = { expressions: Array<Expression>, 
/**
 * Original function name for round-trip preservation (e.g., COALESCE vs IFNULL)
 */
original_name?: string | null, };
