// This file was generated by [ts-rs](https://github.com/Aleph-Alpha/ts-rs). Do not edit this file manually.
import type { Expression } from "./Expression";

/**
 * Represent a unary operation (single operand with a prefix operator).
 *
 * Shared payload for `Not`, `Neg`, and `BitwiseNot` variants.
 */
export type UnaryOp = { 
/**
 * The operand expression.
 */
this: Expression, };
