// This file was generated by [ts-rs](https://github.com/Aleph-Alpha/ts-rs). Do not edit this file manually.
import type { DataType } from "./DataType";
import type { Expression } from "./Expression";

/**
 * Actions for ALTER COLUMN
 */
export type AlterColumnAction = { "SetDataType": { data_type: DataType, 
/**
 * USING expression for type conversion (PostgreSQL)
 */
using: Expression | null, 
/**
 * COLLATE clause (TSQL: ALTER COLUMN col TYPE COLLATE collation_name)
 */
collate?: string | null, } } | { "SetDefault": Expression } | "DropDefault" | "SetNotNull" | "DropNotNull" | { "Comment": string } | "SetVisible" | "SetInvisible";
