// This file was generated by [ts-rs](https://github.com/Aleph-Alpha/ts-rs). Do not edit this file manually.
import type { Expression } from "./Expression";
import type { Identifier } from "./Identifier";

/**
 * UNPIVOT operation
 */
export type Unpivot = { this: Expression, value_column: Identifier, name_column: Identifier, columns: Array<Expression>, alias: Identifier | null, 
/**
 * Whether the value_column was parenthesized in the original SQL
 */
value_column_parenthesized: boolean, 
/**
 * INCLUDE NULLS (true), EXCLUDE NULLS (false), or not specified (None)
 */
include_nulls: boolean | null, 
/**
 * Additional value columns when parenthesized (e.g., (first_half_sales, second_half_sales))
 */
extra_value_columns?: Array<Identifier>, };
