// This file was generated by [ts-rs](https://github.com/Aleph-Alpha/ts-rs). Do not edit this file manually.
import type { AlterColumnAction } from "./AlterColumnAction";
import type { Expression } from "./Expression";
import type { Identifier } from "./Identifier";
import type { TableRef } from "./TableRef";

/**
 * Actions for ALTER VIEW
 */
export type AlterViewAction = { "Rename": TableRef } | { "OwnerTo": Identifier } | { "SetSchema": Identifier } | { "SetAuthorization": string } | { "AlterColumn": { name: Identifier, action: AlterColumnAction, } } | { "AsSelect": Expression } | { "SetTblproperties": Array<[string, string]> } | { "UnsetTblproperties": Array<string> };
