// This file was generated by [ts-rs](https://github.com/Aleph-Alpha/ts-rs). Do not edit this file manually.
import type { Expression } from "./Expression";

/**
 * ARRAY_TRANSFORM / TRANSFORM function (with lambda)
 */
export type ArrayTransformFunc = { this: Expression, transform: Expression, };
