// This file was generated by [ts-rs](https://github.com/Aleph-Alpha/ts-rs). Do not edit this file manually.

/**
 * JSON unique keys constraint variants
 */
export type JsonUniqueKeys = "With" | "Without" | "Shorthand";
