// This file was generated by [ts-rs](https://github.com/Aleph-Alpha/ts-rs). Do not edit this file manually.
import type { WindowFrameBound } from "./WindowFrameBound";
import type { WindowFrameExclude } from "./WindowFrameExclude";
import type { WindowFrameKind } from "./WindowFrameKind";

/**
 * Window frame
 */
export type WindowFrame = { kind: WindowFrameKind, start: WindowFrameBound, end: WindowFrameBound | null, exclude: WindowFrameExclude | null, 
/**
 * Original text of the frame kind keyword (preserves input case, e.g. "range")
 */
kind_text?: string | null, 
/**
 * Original text of the start bound side keyword (e.g. "preceding")
 */
start_side_text?: string | null, 
/**
 * Original text of the end bound side keyword
 */
end_side_text?: string | null, };
