// This file was generated by [ts-rs](https://github.com/Aleph-Alpha/ts-rs). Do not edit this file manually.
import type { Expression } from "./Expression";
import type { QuantifiedOp } from "./QuantifiedOp";

/**
 * ANY / ALL quantified expression
 */
export type QuantifiedExpr = { this: Expression, subquery: Expression, op: QuantifiedOp | null, };
