// This file was generated by [ts-rs](https://github.com/Aleph-Alpha/ts-rs). Do not edit this file manually.
import type { DataType } from "./DataType";
import type { TableRef } from "./TableRef";

/**
 * DROP FUNCTION statement
 */
export type DropFunction = { name: TableRef, parameters: Array<DataType> | null, if_exists: boolean, cascade: boolean, };
