// This file was generated by [ts-rs](https://github.com/Aleph-Alpha/ts-rs). Do not edit this file manually.
import type { Ordered } from "./Ordered";

/**
 * CLUSTER BY clause (Hive/Spark)
 * Combines DISTRIBUTE BY and SORT BY on the same columns
 */
export type ClusterBy = { expressions: Array<Ordered>, };
