// This file was generated by [ts-rs](https://github.com/Aleph-Alpha/ts-rs). Do not edit this file manually.
import type { PseudocolumnType } from "./PseudocolumnType";

/**
 * Pseudocolumn expression (Oracle ROWNUM, ROWID, LEVEL, etc.)
 * These are special identifiers that should not be quoted
 */
export type Pseudocolumn = { kind: PseudocolumnType, };
