// This file was generated by [ts-rs](https://github.com/Aleph-Alpha/ts-rs). Do not edit this file manually.
import type { Expression } from "./Expression";
import type { Identifier } from "./Identifier";
import type { Ordered } from "./Ordered";
import type { WindowFrame } from "./WindowFrame";

/**
 * OVER clause
 */
export type Over = { 
/**
 * Named window reference (e.g., OVER w or OVER (w ORDER BY x))
 */
window_name: Identifier | null, partition_by: Array<Expression>, order_by: Array<Ordered>, frame: WindowFrame | null, alias: Identifier | null, };
