// This file was generated by [ts-rs](https://github.com/Aleph-Alpha/ts-rs). Do not edit this file manually.
import type { Expression } from "./Expression";

/**
 * REGEXP_LIKE function
 */
export type RegexpFunc = { this: Expression, pattern: Expression, flags: Expression | null, };
