// This file was generated by [ts-rs](https://github.com/Aleph-Alpha/ts-rs). Do not edit this file manually.
import type { Expression } from "./Expression";

/**
 * Oracle CONNECT BY clause for hierarchical queries
 */
export type Connect = { 
/**
 * START WITH condition (optional, can come before or after CONNECT BY)
 */
start: Expression | null, 
/**
 * CONNECT BY condition (required, contains PRIOR references)
 */
connect: Expression, 
/**
 * NOCYCLE keyword to prevent infinite loops
 */
nocycle: boolean, };
