// This file was generated by [ts-rs](https://github.com/Aleph-Alpha/ts-rs). Do not edit this file manually.

/**
 * CURRENT_TIMESTAMP
 */
export type CurrentTimestamp = { precision: number | null, 
/**
 * If true, generate SYSDATE instead of CURRENT_TIMESTAMP (Oracle-specific)
 */
sysdate: boolean, };
