// This file was generated by [ts-rs](https://github.com/Aleph-Alpha/ts-rs). Do not edit this file manually.

/**
 * Separator style for named arguments
 */
export type NamedArgSeparator = "DArrow" | "ColonEq" | "Eq";
