// This file was generated by [ts-rs](https://github.com/Aleph-Alpha/ts-rs). Do not edit this file manually.
import type { Expression } from "./Expression";
import type { Ordered } from "./Ordered";

/**
 * NTILE function (DuckDB allows ORDER BY inside)
 */
export type NTileFunc = { 
/**
 * num_buckets is optional to support Databricks NTILE() without arguments
 */
num_buckets?: Expression | null, 
/**
 * DuckDB: NTILE(n ORDER BY col) - order by inside function
 */
order_by?: Array<Ordered> | null, };
