// This file was generated by [ts-rs](https://github.com/Aleph-Alpha/ts-rs). Do not edit this file manually.
import type { Expression } from "./Expression";
import type { Identifier } from "./Identifier";

/**
 * View column definition with optional COMMENT and OPTIONS (BigQuery)
 */
export type ViewColumn = { name: Identifier, comment: string | null, 
/**
 * BigQuery: OPTIONS (key=value, ...) on column
 */
options?: Array<Expression>, };
