// This file was generated by [ts-rs](https://github.com/Aleph-Alpha/ts-rs). Do not edit this file manually.
import type { DataType } from "./DataType";
import type { Expression } from "./Expression";
import type { JsonNullHandling } from "./JsonNullHandling";

/**
 * JSON_OBJECT function
 */
export type JsonObjectFunc = { pairs: Array<[Expression, Expression]>, null_handling: JsonNullHandling | null, with_unique_keys: boolean, returning_type: DataType | null, format_json: boolean, encoding: string | null, 
/**
 * For JSON_OBJECT(*) syntax
 */
star: boolean, };
