// This file was generated by [ts-rs](https://github.com/Aleph-Alpha/ts-rs). Do not edit this file manually.
import type { Expression } from "./Expression";
import type { MatchRecognizeSemantics } from "./MatchRecognizeSemantics";

/**
 * MEASURES expression with optional RUNNING/FINAL semantics
 */
export type MatchRecognizeMeasure = { 
/**
 * The measure expression
 */
this: Expression, 
/**
 * RUNNING or FINAL semantics (Snowflake-specific)
 */
window_frame: MatchRecognizeSemantics | null, };
