// This file was generated by [ts-rs](https://github.com/Aleph-Alpha/ts-rs). Do not edit this file manually.
import type { Expression } from "./Expression";

/**
 * ARRAY_JOIN / ARRAY_TO_STRING function
 */
export type ArrayJoinFunc = { this: Expression, separator: Expression, null_replacement: Expression | null, };
