// This file was generated by [ts-rs](https://github.com/Aleph-Alpha/ts-rs). Do not edit this file manually.
import type { Identifier } from "./Identifier";

/**
 * Actions for ALTER INDEX
 */
export type AlterIndexAction = { "Rename": Identifier } | { "SetTablespace": Identifier } | { "Visible": boolean };
