// This file was generated by [ts-rs](https://github.com/Aleph-Alpha/ts-rs). Do not edit this file manually.
import type { Expression } from "./Expression";

/**
 * Represent a WHERE clause containing a boolean filter predicate.
 */
export type Where = { 
/**
 * The filter predicate expression.
 */
this: Expression, };
