// This file was generated by [ts-rs](https://github.com/Aleph-Alpha/ts-rs). Do not edit this file manually.
import type { Expression } from "./Expression";

/**
 * SequenceProperties
 */
export type SequenceProperties = { increment: Expression | null, minvalue: Expression | null, maxvalue: Expression | null, cache: Expression | null, start: Expression | null, owned: Expression | null, options: Array<Expression>, };
