// This file was generated by [ts-rs](https://github.com/Aleph-Alpha/ts-rs). Do not edit this file manually.
import type { Expression } from "./Expression";

export type WindowFrameBound = "CurrentRow" | "UnboundedPreceding" | "UnboundedFollowing" | { "Preceding": Expression } | { "Following": Expression } | "BarePreceding" | "BareFollowing" | { "Value": Expression };
