// This file was generated by [ts-rs](https://github.com/Aleph-Alpha/ts-rs). Do not edit this file manually.

/**
 * Kind of Teradata index
 */
export type TeradataIndexKind = "NoPrimary" | "Primary" | "PrimaryAmp" | "Unique" | "UniquePrimary" | "Secondary";
