// This file was generated by [ts-rs](https://github.com/Aleph-Alpha/ts-rs). Do not edit this file manually.
import type { Expression } from "./Expression";

export type CollationExpr = { this: Expression, collation: string, 
/**
 * True if the collation was single-quoted in the original SQL (string literal)
 */
quoted: boolean, 
/**
 * True if the collation was double-quoted in the original SQL (identifier)
 */
double_quoted: boolean, };
