// This file was generated by [ts-rs](https://github.com/Aleph-Alpha/ts-rs). Do not edit this file manually.

/**
 * Sample method
 */
export type SampleMethod = "Bernoulli" | "System" | "Block" | "Row" | "Percent" | "Bucket" | "Reservoir";
