// This file was generated by [ts-rs](https://github.com/Aleph-Alpha/ts-rs). Do not edit this file manually.
import type { Expression } from "./Expression";

/**
 * Generated identity column constraint
 */
export type GeneratedAsIdentity = { 
/**
 * True for ALWAYS, False for BY DEFAULT
 */
always: boolean, 
/**
 * ON NULL (only valid with BY DEFAULT)
 */
on_null: boolean, 
/**
 * START WITH value
 */
start: Expression | null, 
/**
 * INCREMENT BY value
 */
increment: Expression | null, 
/**
 * MINVALUE
 */
minvalue: Expression | null, 
/**
 * MAXVALUE
 */
maxvalue: Expression | null, 
/**
 * CYCLE option - Some(true) = CYCLE, Some(false) = NO CYCLE, None = not specified
 */
cycle: boolean | null, };
