// This file was generated by [ts-rs](https://github.com/Aleph-Alpha/ts-rs). Do not edit this file manually.

/**
 * TRUNCATE identity option
 */
export type TruncateIdentity = "Restart" | "Continue";
