// This file was generated by [ts-rs](https://github.com/Aleph-Alpha/ts-rs). Do not edit this file manually.

/**
 * EXCLUDE clause for window frames
 */
export type WindowFrameExclude = "CurrentRow" | "Group" | "Ties" | "NoOthers";
