// This file was generated by [ts-rs](https://github.com/Aleph-Alpha/ts-rs). Do not edit this file manually.
import type { DataType } from "./DataType";
import type { Expression } from "./Expression";

/**
 * CONVERT function (SQL Server style)
 */
export type ConvertFunc = { this: Expression, to: DataType, style: Expression | null, };
