// This file was generated by [ts-rs](https://github.com/Aleph-Alpha/ts-rs). Do not edit this file manually.
import type { Expression } from "./Expression";

/**
 * DataBlocksizeProperty
 */
export type DataBlocksizeProperty = { size: bigint | null, units: Expression | null, minimum: Expression | null, maximum: Expression | null, default: Expression | null, };
