// This file was generated by [ts-rs](https://github.com/Aleph-Alpha/ts-rs). Do not edit this file manually.
import type { Expression } from "./Expression";

/**
 * WithSystemVersioningProperty
 */
export type WithSystemVersioningProperty = { on: Expression | null, this: Expression | null, data_consistency: Expression | null, retention_period: Expression | null, with_: Expression | null, };
