// This file was generated by [ts-rs](https://github.com/Aleph-Alpha/ts-rs). Do not edit this file manually.
import type { Identifier } from "./Identifier";

/**
 * UNCACHE TABLE statement (Spark)
 */
export type Uncache = { 
/**
 * The table to uncache
 */
table: Identifier, 
/**
 * IF EXISTS clause
 */
if_exists: boolean, };
