// This file was generated by [ts-rs](https://github.com/Aleph-Alpha/ts-rs). Do not edit this file manually.
import type { AlterIndexAction } from "./AlterIndexAction";
import type { Identifier } from "./Identifier";
import type { TableRef } from "./TableRef";

/**
 * ALTER INDEX statement
 */
export type AlterIndex = { name: Identifier, table: TableRef | null, actions: Array<AlterIndexAction>, };
