// This file was generated by [ts-rs](https://github.com/Aleph-Alpha/ts-rs). Do not edit this file manually.

/**
 * JSON null handling options
 */
export type JsonNullHandling = "NullOnNull" | "AbsentOnNull";
