// This file was generated by [ts-rs](https://github.com/Aleph-Alpha/ts-rs). Do not edit this file manually.
import type { Expression } from "./Expression";

/**
 * PIVOT alias for aliasing pivot expressions
 * The alias can be an identifier or an expression (for Oracle/BigQuery string concatenation aliases)
 */
export type PivotAlias = { this: Expression, alias: Expression, };
