// This file was generated by [ts-rs](https://github.com/Aleph-Alpha/ts-rs). Do not edit this file manually.
import type { Expression } from "./Expression";

/**
 * RAND function (optional seed, or Teradata RANDOM(lower, upper))
 */
export type Rand = { seed: Expression | null, 
/**
 * Teradata RANDOM lower bound
 */
lower: Expression | null, 
/**
 * Teradata RANDOM upper bound
 */
upper: Expression | null, };
