// This file was generated by [ts-rs](https://github.com/Aleph-Alpha/ts-rs). Do not edit this file manually.
import type { TableRef } from "./TableRef";

/**
 * A table entry in a TRUNCATE statement, with optional ONLY modifier and * suffix
 */
export type TruncateTableEntry = { table: TableRef, 
/**
 * Whether the table has a * suffix (inherit children)
 */
star: boolean, };
