// This file was generated by [ts-rs](https://github.com/Aleph-Alpha/ts-rs). Do not edit this file manually.
import type { Expression } from "./Expression";

/**
 * IndexConstraintOption
 */
export type IndexConstraintOption = { key_block_size: Expression | null, using: Expression | null, parser: Expression | null, comment: Expression | null, visible: Expression | null, engine_attr: Expression | null, secondary_engine_attr: Expression | null, };
