// This file was generated by [ts-rs](https://github.com/Aleph-Alpha/ts-rs). Do not edit this file manually.
import type { Identifier } from "./Identifier";

/**
 * AFTER MATCH SKIP behavior
 */
export type MatchRecognizeAfter = "PastLastRow" | "ToNextRow" | { "ToFirst": Identifier } | { "ToLast": Identifier };
