// This file was generated by [ts-rs](https://github.com/Aleph-Alpha/ts-rs). Do not edit this file manually.

/**
 * Type of scope in a SQL query
 */
export type ScopeType = "Root" | "Subquery" | "DerivedTable" | "Cte" | "SetOperation" | "Udtf";
