// This file was generated by [ts-rs](https://github.com/Aleph-Alpha/ts-rs). Do not edit this file manually.

/**
 * Types of properties in CREATE FUNCTION for tracking their original order
 */
export type FunctionPropertyKind = "Set" | "As" | "Language" | "Determinism" | "NullInput" | "Security" | "SqlDataAccess" | "Options" | "Environment";
