// This file was generated by [ts-rs](https://github.com/Aleph-Alpha/ts-rs). Do not edit this file manually.
import type { TableRef } from "./TableRef";
import type { TypeDefinition } from "./TypeDefinition";

/**
 * CREATE TYPE statement
 */
export type CreateType = { name: TableRef, definition: TypeDefinition, if_not_exists: boolean, };
