// This file was generated by [ts-rs](https://github.com/Aleph-Alpha/ts-rs). Do not edit this file manually.
import type { Expression } from "./Expression";

/**
 * AT TIME ZONE expression for timezone conversion
 */
export type AtTimeZone = { 
/**
 * The expression to convert
 */
this: Expression, 
/**
 * The target timezone
 */
zone: Expression, };
