// This file was generated by [ts-rs](https://github.com/Aleph-Alpha/ts-rs). Do not edit this file manually.

/**
 * Var - Simple variable reference (for SQL variables, keywords as values)
 */
export type Var = { this: string, };
