// This file was generated by [ts-rs](https://github.com/Aleph-Alpha/ts-rs). Do not edit this file manually.
import type { Expression } from "./Expression";

/**
 * CEIL function with optional decimals and time unit (Druid: CEIL(time TO unit))
 */
export type CeilFunc = { this: Expression, decimals?: Expression | null, 
/**
 * Time unit for Druid-style CEIL(time TO unit) syntax
 */
to?: Expression | null, };
