// This file was generated by [ts-rs](https://github.com/Aleph-Alpha/ts-rs). Do not edit this file manually.
import type { Identifier } from "./Identifier";

/**
 * Trigger REFERENCING clause
 */
export type TriggerReferencing = { old_table: Identifier | null, new_table: Identifier | null, old_row: Identifier | null, new_row: Identifier | null, };
