// This file was generated by [ts-rs](https://github.com/Aleph-Alpha/ts-rs). Do not edit this file manually.
import type { CopyParameter } from "./CopyParameter";
import type { Expression } from "./Expression";

/**
 * PUT statement (Snowflake)
 */
export type PutStmt = { 
/**
 * Source file path
 */
source: string, 
/**
 * Whether source was quoted in the original SQL
 */
source_quoted: boolean, 
/**
 * Target stage
 */
target: Expression, 
/**
 * PUT parameters
 */
params: Array<CopyParameter>, };
