// This file was generated by [ts-rs](https://github.com/Aleph-Alpha/ts-rs). Do not edit this file manually.
import type { TeradataIndexKind } from "./TeradataIndexKind";

/**
 * Teradata index specification for CREATE TABLE
 */
export type TeradataIndex = { 
/**
 * Index kind: NoPrimary, Primary, PrimaryAmp, Unique, UniquePrimary
 */
kind: TeradataIndexKind, 
/**
 * Optional index name
 */
name: string | null, 
/**
 * Optional column list
 */
columns: Array<string>, };
