// This file was generated by [ts-rs](https://github.com/Aleph-Alpha/ts-rs). Do not edit this file manually.

/**
 * Type of column constraint for tracking order
 */
export type ConstraintType = "NotNull" | "Null" | "PrimaryKey" | "Unique" | "Default" | "AutoIncrement" | "Collate" | "Comment" | "References" | "Check" | "GeneratedAsIdentity" | "Tags" | "ComputedColumn" | "GeneratedAsRow" | "OnUpdate" | "Path" | "Encode";
