// This file was generated by [ts-rs](https://github.com/Aleph-Alpha/ts-rs). Do not edit this file manually.
import type { Identifier } from "./Identifier";

/**
 * Index column specification
 */
export type IndexColumn = { column: Identifier, desc: boolean, 
/**
 * Explicit ASC keyword was present
 */
asc: boolean, nulls_first: boolean | null, 
/**
 * PostgreSQL operator class (e.g., varchar_pattern_ops, public.gin_trgm_ops)
 */
opclass?: string | null, };
