// This file was generated by [ts-rs](https://github.com/Aleph-Alpha/ts-rs). Do not edit this file manually.
import type { Expression } from "./Expression";
import type { Ordered } from "./Ordered";

/**
 * PERCENTILE_CONT / PERCENTILE_DISC function
 */
export type PercentileFunc = { this: Expression, percentile: Expression, order_by: Array<Ordered> | null, filter: Expression | null, };
