// This file was generated by [ts-rs](https://github.com/Aleph-Alpha/ts-rs). Do not edit this file manually.
import type { ComputedColumn } from "./ComputedColumn";
import type { Expression } from "./Expression";
import type { ForeignKeyRef } from "./ForeignKeyRef";
import type { GeneratedAsIdentity } from "./GeneratedAsIdentity";
import type { GeneratedAsRow } from "./GeneratedAsRow";
import type { Identifier } from "./Identifier";
import type { Tags } from "./Tags";

/**
 * Column-level constraint
 */
export type ColumnConstraint = "NotNull" | "Null" | "Unique" | "PrimaryKey" | { "Default": Expression } | { "Check": Expression } | { "References": ForeignKeyRef } | { "GeneratedAsIdentity": GeneratedAsIdentity } | { "Collate": Identifier } | { "Comment": string } | { "Tags": Tags } | { "ComputedColumn": ComputedColumn } | { "GeneratedAsRow": GeneratedAsRow } | { "Path": Expression };
