// This file was generated by [ts-rs](https://github.com/Aleph-Alpha/ts-rs). Do not edit this file manually.

/**
 * Automatically converts unit arg into a var.
 */
export type TimeUnit = { unit: string | null, };
