// This file was generated by [ts-rs](https://github.com/Aleph-Alpha/ts-rs). Do not edit this file manually.

/**
 * ON COMMIT behavior for temporary tables
 */
export type OnCommit = "PreserveRows" | "DeleteRows";
