// This file was generated by [ts-rs](https://github.com/Aleph-Alpha/ts-rs). Do not edit this file manually.
import type { Expression } from "./Expression";
import type { Identifier } from "./Identifier";
import type { NamedArgSeparator } from "./NamedArgSeparator";

/**
 * Named argument in function call: name => value or name := value
 */
export type NamedArgument = { name: Identifier, value: Expression, 
/**
 * The separator used: `=>`, `:=`, or `=`
 */
separator: NamedArgSeparator, };
