// This file was generated by [ts-rs](https://github.com/Aleph-Alpha/ts-rs). Do not edit this file manually.

export type DateTimeField = "Year" | "Month" | "Day" | "Hour" | "Minute" | "Second" | "Millisecond" | "Microsecond" | "DayOfWeek" | "DayOfYear" | "Week" | { "WeekWithModifier": string } | "Quarter" | "Epoch" | "Timezone" | "TimezoneHour" | "TimezoneMinute" | "Date" | "Time" | { "Custom": string };
