// This file was generated by [ts-rs](https://github.com/Aleph-Alpha/ts-rs). Do not edit this file manually.
import type { Expression } from "./Expression";

/**
 * FLOOR function with optional scale and time unit (Druid: FLOOR(time TO unit))
 */
export type FloorFunc = { this: Expression, scale: Expression | null, 
/**
 * Time unit for Druid-style FLOOR(time TO unit) syntax
 */
to?: Expression | null, };
