// This file was generated by [ts-rs](https://github.com/Aleph-Alpha/ts-rs). Do not edit this file manually.
import type { Identifier } from "./Identifier";

/**
 * RollupIndex - A single rollup index: name(col1, col2)
 */
export type RollupIndex = { name: Identifier, expressions: Array<Identifier>, };
