// This file was generated by [ts-rs](https://github.com/Aleph-Alpha/ts-rs). Do not edit this file manually.
import type { Expression } from "./Expression";

/**
 * JSONExtract
 */
export type JSONExtract = { this: Expression, expression: Expression, only_json_types: Expression | null, expressions: Array<Expression>, variant_extract: Expression | null, json_query: Expression | null, option: Expression | null, quote: Expression | null, on_condition: Expression | null, requires_json: Expression | null, };
