// This file was generated by [ts-rs](https://github.com/Aleph-Alpha/ts-rs). Do not edit this file manually.
import type { Expression } from "./Expression";

/**
 * Named parameter in EXEC statement: @name=value
 */
export type ExecuteParameter = { 
/**
 * Parameter name (including @)
 */
name: string, 
/**
 * Parameter value
 */
value: Expression, };
