// This file was generated by [ts-rs](https://github.com/Aleph-Alpha/ts-rs). Do not edit this file manually.
import type { Expression } from "./Expression";

/**
 * STREAM definition (Snowflake) - for change data capture
 */
export type Stream = { this: Expression, on: Expression | null, show_initial_rows: boolean | null, };
