// This file was generated by [ts-rs](https://github.com/Aleph-Alpha/ts-rs). Do not edit this file manually.
import type { DateTimeField } from "./DateTimeField";
import type { Expression } from "./Expression";

/**
 * LAST_DAY function with optional date part (for BigQuery granularity like WEEK(SUNDAY))
 */
export type LastDayFunc = { this: Expression, 
/**
 * Optional date part for granularity (e.g., MONTH, YEAR, WEEK(SUNDAY))
 */
unit?: DateTimeField | null, };
