// This file was generated by [ts-rs](https://github.com/Aleph-Alpha/ts-rs). Do not edit this file manually.
import type { Expression } from "./Expression";

/**
 * DISTRIBUTE BY clause (Hive/Spark)
 * Controls how rows are distributed across reducers
 */
export type DistributeBy = { expressions: Array<Expression>, };
