// This file was generated by [ts-rs](https://github.com/Aleph-Alpha/ts-rs). Do not edit this file manually.
import type { IntervalUnit } from "./IntervalUnit";

/**
 * Interval span for ranges like HOUR TO SECOND
 */
export type IntervalSpan = { 
/**
 * Start unit (e.g., HOUR)
 */
this: IntervalUnit, 
/**
 * End unit (e.g., SECOND)
 */
expression: IntervalUnit, };
