// This file was generated by [ts-rs](https://github.com/Aleph-Alpha/ts-rs). Do not edit this file manually.
import type { DatabaseOption } from "./DatabaseOption";
import type { Expression } from "./Expression";
import type { Identifier } from "./Identifier";

/**
 * CREATE DATABASE statement
 */
export type CreateDatabase = { name: Identifier, if_not_exists: boolean, options: Array<DatabaseOption>, 
/**
 * Snowflake CLONE source
 */
clone_from: Identifier | null, 
/**
 * AT/BEFORE clause for time travel (Snowflake)
 */
at_clause: Expression | null, };
