// This file was generated by [ts-rs](https://github.com/Aleph-Alpha/ts-rs). Do not edit this file manually.
import type { Expression } from "./Expression";
import type { Identifier } from "./Identifier";

/**
 * Dot access (struct.field)
 */
export type DotAccess = { this: Expression, field: Identifier, };
