// This file was generated by [ts-rs](https://github.com/Aleph-Alpha/ts-rs). Do not edit this file manually.
import type { Expression } from "./Expression";

/**
 * Represent an EXISTS predicate (`EXISTS (SELECT ...)` or `NOT EXISTS (...)`).
 */
export type Exists = { 
/**
 * The subquery expression.
 */
this: Expression, 
/**
 * Whether this is NOT EXISTS.
 */
not: boolean, };
