// This file was generated by [ts-rs](https://github.com/Aleph-Alpha/ts-rs). Do not edit this file manually.
import type { Expression } from "./Expression";

/**
 * Represent a GROUP BY clause with optional ALL/DISTINCT and WITH TOTALS modifiers.
 *
 * The `expressions` list may contain plain columns, ordinal positions,
 * ROLLUP/CUBE/GROUPING SETS expressions, or the special empty-set `()`.
 */
export type GroupBy = { 
/**
 * The grouping expressions.
 */
expressions: Array<Expression>, 
/**
 * GROUP BY modifier: Some(true) = ALL, Some(false) = DISTINCT, None = no modifier
 */
all: boolean | null, 
/**
 * ClickHouse: WITH TOTALS modifier
 */
totals: boolean, };
