// This file was generated by [ts-rs](https://github.com/Aleph-Alpha/ts-rs). Do not edit this file manually.

/**
 * Semantics for MEASURES in MATCH_RECOGNIZE
 */
export type MatchRecognizeSemantics = "Running" | "Final";
