// This file was generated by [ts-rs](https://github.com/Aleph-Alpha/ts-rs). Do not edit this file manually.
import type { Expression } from "./Expression";
import type { Identifier } from "./Identifier";
import type { TableRef } from "./TableRef";
import type { TriggerBody } from "./TriggerBody";
import type { TriggerEvent } from "./TriggerEvent";
import type { TriggerForEach } from "./TriggerForEach";
import type { TriggerReferencing } from "./TriggerReferencing";
import type { TriggerTiming } from "./TriggerTiming";

/**
 * CREATE TRIGGER statement
 */
export type CreateTrigger = { name: Identifier, table: TableRef, timing: TriggerTiming, events: Array<TriggerEvent>, for_each: TriggerForEach, when: Expression | null, body: TriggerBody, or_replace: boolean, constraint: boolean, deferrable: boolean | null, initially_deferred: boolean | null, referencing: TriggerReferencing | null, };
